module Devpad
  module API
    class Repository < OpenStruct
      def self.all(params = {})
        @response = Devpad::API::Client.get("/accounts/#{Devpad::Account.current!}/repositories", params)
        @response['data'].map do |item|
          new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
        end
      end

      def self.create(params = {})
        @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/repositories", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.clone(params = {})
        repository_id = params.delete :id
        @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/clone", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.close(params = {})
        repository_id = params.delete :id
        @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/close", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.protect(params = {})
        repository_id = params.delete :id
        @response = Devpad::API::Client.put("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/protect", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.unprotect(params = {})
        repository_id = params.delete :id
        @response = Devpad::API::Client.delete("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/protect", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.download(params = {})
        format = params.delete(:format) || 'zip'
        repository_id = params.delete(:id)
        path_to_save = params.delete(:to)
        Devpad::API::Client.download("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/download.#{format}", path_to_save, params)
      end

      class Branch < OpenStruct
        def self.all(params = {})
          repository_id = params.delete :id
          @response = Devpad::API::Client.get("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/branches", params)
          @response['data'].map do |item|
            new(item['attributes'].merge('id' => item['id']))
          end
        end
      end

      class Commit < OpenStruct
        def self.all(params = {})
          repository_id = params.delete :id
          @response = Devpad::API::Client.get("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/commits", params)
          @response['data'].map do |item|
            new(item['attributes'].merge('id' => item['id']))
          end
        end
      end

      class User < OpenStruct
        def self.update(params = {})
          repository_id = params.delete :id
          @response = Devpad::API::Client.put("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/user", params)
          item = @response['data']
          Repository.new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
        end
      end

      class Link < OpenStruct
        def self.create(params = {})
          repository_id = params.delete :id
          @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/link", params)
          item = @response['data']
          new(item['attributes'].merge('id' => item['id']))
        end
      end

      class Invitation < OpenStruct
        def self.create(params = {})
          repository_id = params.delete :id
          @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/repositories/#{repository_id}/invitation", params)
          item = @response['data']
          new(item['attributes'].merge('id' => item['id']))
        end
      end
    end
  end
end
