module Devpad
  module API
    class Review < OpenStruct
      def self.all(params = {})
        @response = Devpad::API::Client.get("/accounts/#{Devpad::Account.current!}/reviews", params)
        @response['data'].map do |item|
          new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
        end
      end

      def self.create(params = {})
        @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/reviews", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.clone(params = {})
        review_id = params.delete :id
        @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/clone", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      def self.close(params = {})
        review_id = params.delete :id
        @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/close", params)
        item = @response['data']
        new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
      end

      class User < OpenStruct
        def self.update(params = {})
          review_id = params.delete :id
          @response = Devpad::API::Client.put("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/user", params)
          item = @response['data']
          Pad.new(item['attributes'].merge('id' => item['id']).merge('user_name' => item.dig('relationships', 'user', 'data', 'attributes', 'name')))
        end
      end

      class Link < OpenStruct
        def self.create(params = {})
          review_id = params.delete :id
          @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/link", params)
          item = @response['data']
          new(item['attributes'].merge('id' => item['id']))
        end
      end

      class Invitation < OpenStruct
        def self.create(params = {})
          review_id = params.delete :id
          @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/invitation", params)
          item = @response['data']
          new(item['attributes'].merge('id' => item['id']))
        end
      end

      class File < OpenStruct
        def self.all(params = {})
          review_id = params.delete :id
          @response = Devpad::API::Client.get("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/files", params)
          @response['data'].map do |item|
            language = item.dig('relationships', 'language', 'data', 'attributes', 'name')
            new(item['attributes'].merge('id' => item['id'], language: language))
          end
        end

        def self.create(params = {})
          review_id = params.delete :id
          @response = Devpad::API::Client.post("/accounts/#{Devpad::Account.current!}/reviews/#{review_id}/files", params)
          item = @response['data']
          new(item['attributes'].merge('id' => item['id']))
        end
      end
    end
  end
end
