module Devpad
  module CLI
    class Pad < Thor
      option :page, type: :numeric, default: 1, aliases: '-p'
      options active: :boolean, closed: :boolean, template: :boolean
      desc 'list', 'List of available test suites'
      def list
        if options[:active].present?
          status = 'active'
        elsif options[:closed].present?
          status = 'closed'
        else
          status = nil
        end
        params = options.slice('template', 'page').merge('status' => status)
        Devpad::CLI::Commands::Pad::List.new(shell: shell).execute(params)
      end

      option :name, type: :string, aliases: '-n'
      option :description, type: :string, aliases: '-d'
      option :template, type: :boolean, default: false, aliases: '-t'
      option :language, type: :string, aliases: '-l'
      options :'with-link' => :boolean
      desc 'create', 'Create a pad'
      def create
        params = {
          pad: {
            name: options[:name],
            description: options[:description],
            template: options[:template],
            language_id: options[:language]
          }
        }
        pad_id = Devpad::CLI::Commands::Pad::Create.new(shell: shell).execute(params)
        if options[:'with-link']
          Devpad::CLI::Commands::Pad::Link.new(shell: shell).execute(id: pad_id)
        end
      end

      option :name, type: :string, aliases: '-n'
      option :description, type: :string, aliases: '-d'
      options template: :boolean, content: :boolean, initial_code: :boolean, current_code: :boolean, containers: :boolean, :'with-link' => :boolean
      desc 'create', 'Create a pad'
      def clone(pad_id)
        pad_clone = {
          name: options[:name],
          description: options[:description],
          template: options[:template],
          content: options[:content],
          initial_code: options[:initial_code],
          current_code: options[:current_code],
          containers: options[:containers]
        }
        params = {
          id: pad_id,
          pad_clone: pad_clone
        }
        new_pad_id = Devpad::CLI::Commands::Pad::Clone.new(shell: shell).execute(params)
        if options[:'with-link']
          Devpad::CLI::Commands::Pad::Link.new(shell: shell).execute(id: new_pad_id)
        end
      end

      desc 'user [PAD]', 'Set a user'
      def user(pad_id)
        Devpad::CLI::Commands::Pad::User.new(shell: shell).execute(id: pad_id)
      end

      option :name, type: :string, required: true
      option :email, type: :string, required: true
      desc 'invitation [PAD]', 'Invite a participant'
      def invitation(pad_id)
        params = {
          id: pad_id,
          pad_invitation: {
            email_attributes: {
              name: options[:name], address: options[:email]
            }
          }
        }
        Devpad::CLI::Commands::Pad::Invitation.new(shell: shell).execute(params)
      end

      desc 'link [PAD]', 'Create a link'
      def link(pad_id)
        Devpad::CLI::Commands::Pad::Link.new(shell: shell).execute(id: pad_id)
      end

      desc 'close [PAD]', 'Close a pad'
      def close(pad_id)
        Devpad::CLI::Commands::Pad::Close.new(shell: shell).execute(id: pad_id)
      end
    end
  end
end
