module Devpad
  module CLI
    class Repository < Thor
      option :page, type: :numeric, default: 1
      options active: :boolean, closed: :boolean, template: :boolean
      desc 'list', 'List of available test suites'
      def list
        if options[:active].present?
          status = 'active'
        elsif options[:closed].present?
          status = 'closed'
        else
          status = nil
        end
        params = options.slice('template', 'page').merge('status' => status)
        Devpad::CLI::Commands::Repository::List.new(shell: shell).execute(params)
      end

      option :name, type: :string
      option :description, type: :string
      options template: :boolean, single: :boolean
      desc 'create', 'Create a repository'
      def create
        params = {
          repository: {
            name: options[:name],
            description: options[:description],
            template: options[:template],
            containers_single_use_mode: options[:single]
          }
        }
        repository_id = Devpad::CLI::Commands::Repository::Create.new(shell: shell).execute(params)
        if options[:'with-link']
          Devpad::CLI::Commands::Repository::Link.new(shell: shell).execute(id: repository_id)
        end
      end

      option :name, type: :string
      option :description, type: :string
      option :branch, type: :string
      options commits: :boolean, tasks: :boolean, template: :boolean,
        containers: :boolean, images: :boolean, single: :boolean
      desc 'clone [REPOSITORY]', 'Clone a repository'
      def clone(repository_id)
        review_clone = {
          name: options[:name],
          description: options[:description],
          commits: options[:commits],
          tasks: options[:tasks],
          template: options[:template],
          containers: options[:containers],
          images: options[:images],
          containers_single_use_mode: options[:single],
          branch: options[:branch]
        }
        params = {
          id: repository_id,
          repository_clone: review_clone
        }
        new_repository_id = Devpad::CLI::Commands::Repository::Clone.new(shell: shell).execute(params)
        if options[:'with-link']
          Devpad::CLI::Commands::Repository::Link.new(shell: shell).execute(id: new_repository_id)
        end
      end

      desc 'protect [REPOSITORY]', 'Protect a repository'
      def protect(repository_id)
        Devpad::CLI::Commands::Repository::Protect.new(shell: shell).execute(id: repository_id)
      end

      desc 'unprotect [REPOSITORY]', 'Unprotect a repository'
      def unprotect(repository_id)
        Devpad::CLI::Commands::Repository::Unprotect.new(shell: shell).execute(id: repository_id)
      end

      desc 'user [REPOSITORY]', 'Set a user'
      def user(repository_id)
        Devpad::CLI::Commands::Repository::User.new(shell: shell).execute(id: repository_id)
      end

      option :name, type: :string, required: true
      option :email, type: :string, required: true
      desc 'invitation [REPOSITORY]', 'Invite a participant'
      def invitation(repository_id)
        params = {
          id: repository_id,
          repository_invitation: {
            email_attributes: {
              name: options[:name], address: options[:email]
            }
          }
        }
        Devpad::CLI::Commands::Repository::Invitation.new(shell: shell).execute(params)
      end

      desc 'link [REPOSITORY]', 'Create a link'
      def link(repository_id)
        Devpad::CLI::Commands::Repository::Link.new(shell: shell).execute(id: repository_id)
      end

      option :sha, type: :string, default: 'master'
      option :to, type: :string, default: nil
      desc 'download [REPOSITORY]', 'Download a repository'
      def download(repository_id)
        Devpad::CLI::Commands::Repository::Download.new(shell: shell).execute(id: repository_id, to: options[:to], sha: options[:sha])
      end

      desc 'close [REPOSITORY]', 'Close a repository'
      def close(repository_id)
        Devpad::CLI::Commands::Repository::Close.new(shell: shell).execute(id: repository_id)
      end

      option :page, type: :numeric, default: 1
      desc 'branches [REPOSITORY]', 'List a repository branches'
      def branches(repository_id)
        params = { id: repository_id }
        Devpad::CLI::Commands::Repository::Branch::List.new(shell: shell).execute(params.merge('page' => options[:page]))
      end

      option :page, type: :numeric, default: 1
      desc 'commits [REPOSITORY]', 'List a repository commits'
      def commits(repository_id)
        params = { id: repository_id }
        Devpad::CLI::Commands::Repository::Commit::List.new(shell: shell).execute(params.merge('page' => options[:page]))
      end
    end
  end
end
