module Devpad
  module CLI
    class Review < Thor
      option :page, type: :numeric, default: 1, aliases: '-p'
      options active: :boolean, closed: :boolean, template: :boolean
      desc 'list', 'List of available reviews'
      def list
        if options[:active].present?
          status = 'active'
        elsif options[:closed].present?
          status = 'closed'
        else
          status = nil
        end
        params = options.slice('template', 'page').merge('status' => status)
        Devpad::CLI::Commands::Review::List.new(shell: shell).execute(params)
      end

      option :name, type: :string
      option :description, type: :string
      options template: :boolean, single: :boolean
      desc 'create', 'Create a review'
      def create
        params = {
          review: {
            name: options[:name],
            description: options[:description],
            template: options[:template],
            containers_single_use_mode: options[:single],
          }
        }
        review_id = Devpad::CLI::Commands::Review::Create.new(shell: shell).execute(params)
        if options[:'with-link']
          Devpad::CLI::Commands::Review::Link.new(shell: shell).execute(id: review_id)
        end
      end

      option :name, type: :string, aliases: '-n'
      option :description, type: :string, aliases: '-d'
      options comments: :boolean, highlights: :boolean, template: :boolean, containers: :boolean, images: :boolean, :'with-link' => :boolean, single: :boolean
      desc 'create', 'Create a review'
      def clone(review_id)
        review_clone = {
          name: options[:name],
          description: options[:description],
          comments: options[:comments],
          highlights: options[:highlights],
          template: options[:template],
          containers: options[:containers],
          images: options[:images],
          containers_single_use_mode: options[:single]
        }
        params = {
          id: review_id,
          review_clone: review_clone
        }
        new_review_id = Devpad::CLI::Commands::Review::Clone.new(shell: shell).execute(params)
        if options[:'with-link']
          Devpad::CLI::Commands::Review::Link.new(shell: shell).execute(id: new_review_id)
        end
      end

      desc 'link [REVIEW]', 'Create a link'
      def link(review_id)
        Devpad::CLI::Commands::Review::Link.new(shell: shell).execute(id: review_id)
      end

      desc 'close [REVIEW]', 'Close a review'
      def close(review_id)
        Devpad::CLI::Commands::Review::Close.new(shell: shell).execute(id: review_id)
      end

      option :name, type: :string, default: nil
      option :language, type: :string, default: nil
      desc 'upload [REVIEW] [FILE]', 'Upload a file'
      def upload(review_id, path)
        if File.exist?(path)
          name = options[:name] || File.basename(path)
          if options[:language].present?
            language_id = Integer(options[:language]) rescue nil
            if language_id.nil?
              language_id = Linguist::Language.find_by_name(options[:language])&.language_id
            end
          else
            language_id = Linguist::Language.find_by_extension(File.extname(name))&.first&.language_id
          end
          Devpad::CLI::Commands::Review::File::Create.new(shell: shell).execute(
            id: review_id, review_file: { name: name, content: File.read(path), language_id: language_id }
          )
        else
          shell.say "File `#{path}' does not exist"
        end
      end

      option :page, type: :numeric, default: 1
      desc 'files [REVIEW]', 'Display list of files'
      def files(review_id)
        params = { 'id': review_id }
        params['page'] = options[:page]
        Devpad::CLI::Commands::Review::File::List.new(shell: shell).execute(params)
      end

      desc 'user [REVIEW]', 'Set a user'
      def user(review_id)
        Devpad::CLI::Commands::Review::User.new(shell: shell).execute(id: review_id)
      end

      option :name, type: :string, required: true
      option :email, type: :string, required: true
      desc 'invitation [REVIEW]', 'Invite a reviewer'
      def invitation(review_id)
        params = {
          id: review_id,
          review_invitation: {
            email_attributes: {
              name: options[:name], address: options[:email]
            }
          }
        }
        Devpad::CLI::Commands::Review::Invitation.new(shell: shell).execute(params)
      end
    end
  end
end
