require 'faraday'

module Devpad
  class Client
    attr_accessor :endpoint,
                  :api_format,
                  :user_agent,
                  :api_version,
                  :user_api_key

    def initialize(options = {})
      Devpad::Configuration::VALID_CLIENT_ATTRIBUTES.each do |key|
        send("#{key}=", options[key])
      end
    end

    def connection(options = {})
      api_version = options.delete(:api_version) || self.api_version
      api_format  = options.delete(:api_format)  || :json

      options = {
        url: self.endpoint,
        params: {},
        headers: {
          accept: "application/vnd.devpad.v#{api_version}+#{api_format}",
          user_agent: self.user_agent,
          x_gem_version: Devpad::VERSION,
        }.merge(options.delete(:headers) || {})
      }.merge(options)

      Faraday.new(options) do |c|
        c.request :multipart
        c.request :url_encoded
        c.adapter :net_http
      end
    end

    def download(endpoint, to, params = {})
      response = connection.method(:get).call(endpoint, params)
      open(to, "wb") do |file|
        file.write(response.body)
      end
    end

    def get(endpoint, params = {})
      json_response(:get, endpoint_with_host(endpoint), params)
    end

    def post(endpoint, params = {})
      json_response(:post, endpoint_with_host(endpoint), params)
    end

    private

    def make_request(method, endpoint, params = {})
      connection.method(method).call(endpoint, params) do |request|
        if self.user_api_key.present?
          request.headers['Authorization'] = self.user_api_key
        end
      end
    end

    def json_response(method, endpoint, params = {})
      json = JSON.parse(make_request(method, endpoint, params).body)
      if json['errors']
        puts json['errors'].map { |item| item['detail'] }.join("\n")
        exit!
      end
      json
    end

    def endpoint_with_host(path)
      "#{endpoint}#{path}"
    end
  end
end
